from fastapi import FastAPI
import json
from fastapi.middleware.cors import CORSMiddleware
import uvicorn

class Database:

    def __init__(self, db_name=None):
        if db_name == None:
            db_name = "database.json"
        self.db_name = db_name
        
        #jezeli uda nam sie stworzyc, to znaczy ze nie ma jeszcze bazy danych
        try:
            with open(self.db_name, "x") as db:
                self.database = {"Players": []}
        except FileExistsError:
            self.database = self._loadDatabase()

    
    def _loadDatabase(self):
        with open(self.db_name, "r") as db:
            data = json.load(db)
        return data

    # zwraca indeks użytkownika lub -1 jeżeli nie istnieje
    def _getUserIndex(self, nick):
        for i in range(len(self.database["Players"])):
            if self.database["Players"][i]["nick"] == nick:
                return i
        return -1

    #jeżeli chcemy dodać użytkownika najpierw musimy sprawdzic czy taki przypadkiem nie istnieje
    def _addUserIfDoesntExist(self, nick):
        self.database["Players"].append({"nick": nick, "games": []})
        return len(self.database["Players"])-1

    def addUserData(self, nick, mode, timestamp, result):
        #jezeli nie istnieje, tworzymy dla niego "Szkielet"
        nick_idx = self._getUserIndex(nick)
        if nick_idx == -1:
            nick_idx = self._addUserIfDoesntExist(nick)
        self.database["Players"][nick_idx]["games"].append({"timestamp": timestamp, "mode": mode, "result": result})

        data_json = json.dumps(self.database, indent =4)

        with open(self.db_name, "w") as outfile:
            outfile.write(data_json)

        return 1

    def getUserData(self, nick):
        user_idx = self._getUserIndex(nick)
        if user_idx == -1:
            return "User doesn't exist"
        return self.database["Players"][user_idx]

    def getBestUser(self, mode=None):
        if mode ==None:
            mode = 1
        best_nick = None
        best_result = None
        for i in range (len(self.database["Players"])):
             for game in self.database["Players"][i]["games"]:
                 if game['mode'] == mode:
                    if best_nick == None:
                         best_nick = self.database["Players"][i]["nick"]
                         best_result = game["result"]
                    elif game["result"] > best_result:
                        best_nick = self.database["Players"][i]["nick"]
                        best_result = game["result"]
        if best_nick == None:
            return "None", 0
        return best_nick, best_result

    def getUserBest(self, nick, mode=None):
        best_score = 0
        if mode ==None:
            mode = 1
        for i in range (len(self.database["Players"])):
            if self.database["Players"][i]["nick"] == nick:
                for game in self.database["Players"][i]["games"]:
                    if game['mode'] == mode and game["result"] >= best_score:
                        best_score = game["result"]
                break

        return nick, best_score



if __name__ == "__main__":
    app = FastAPI()
    app.database = Database()

    app.origins = ["*"]

    app.add_middleware(
        CORSMiddleware,
        allow_origins=app.origins,
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )

    @app.get("/")
    async def root():
        return {"message": "hello World"}

    @app.get("/user_data/{nick}")
    async def read_user(nick: str):
        return {nick: app.database.getUserData(nick)}

    @app.get("/best")
    async def best_user():
        best_nick, best_result = app.database.getBestUser()
        return {"nick": best_nick, "result": best_result}

    @app.get("/best/{nick}")
    async def user_best(nick: str):
        nick, best_result = app.database.getUserBest(nick)
        return {"nick": nick, "result": best_result}

    @app.post("/user_data/")
    async def add_result(nick: str, timestamp: int, mode: int, result: int):
        return app.database.addUserData(nick, mode, timestamp, result)
        
    uvicorn.run(app, host="0.0.0.0", port=8000)


